Attribute VB_Name = "Module1"
Option Explicit
'
' Copyright  1997-2000 Brad Martinez, http://www.mvps.org
'
Public Enum ComponentSafetyCmds
  cscMark = 0
  cscUnmark = 1
  cscIsMarked = 2
End Enum

Public Const MAX_PATH = 260

Declare Function SendMessage Lib "user32" Alias "SendMessageA" _
                            (ByVal hwnd As Long, _
                            ByVal wMsg As Long, _
                            ByVal wParam As Long, _
                            lParam As Any) As Long   ' <---

Public Type POINTAPI   ' pt
  x As Long
  y As Long
End Type

Declare Function GetCursorPos Lib "user32" (lpPoint As POINTAPI) As Boolean
Declare Function ScreenToClient Lib "user32" (ByVal hwnd As Long, lpPoint As Any) As Long  ' lpPoint As POINTAPI) As Long

' ======================================================================
' listview

' using IE3 definitions

' ======================================================================
' CLSIDs

' GUID's are 38 chars long, including the brackets, make room for the null terminator & some slop...
Public Const MAX_GUID = 40   ' user-defined

' Definition names (but not values) are defined in Objsafe.h and Objsafe.idl
' Values are found under HKCR\Component Categories
Public Const CATID_SafeForInitializing = "{7DD95802-9882-11CF-9FA9-00AA006C42C4}"
Public Const CATID_SafeForScripting = "{7DD95801-9882-11CF-9FA9-00AA006C42C4}"

Public Type GUID          ' 16 bytes (128 bits)
  dwData1 As Long      ' 4 bytes
  wData2 As Integer     ' 2 bytes
  wData3 As Integer     ' 2 bytes
  abData4(7) As Byte   ' 8 bytes, zero based
End Type

Public Const S_OK = 0   ' HRESULT indicating success

' Both return HRESULTs, S_OK on success.
Declare Function CLSIDFromProgID Lib "ole32" (ByVal lpszProgID As String, pclsid As GUID) As Long
Declare Function CLSIDFromString Lib "ole32" (ByVal lpsz As String, pclsid As GUID) As Long

' rguid:     interface identifier to be converted.
' lpsz:       pointer to the resulting string on return.
' cbMax:  maximum size the returned string is expected to be.
' Rtns the number of chars copied to the lpsz param, including the null terminator.
Declare Function StringFromGUID2 Lib "ole32" (rguid As GUID, lpsz As Any, ByVal cbMax As Long) As Long

' ======================================================================
' Registry

Public Type FILETIME   ' ft
  dwLowDateTime As Long
  dwHighDateTime As Long
End Type

Declare Function RegOpenKeyEx Lib "advapi32.dll" Alias "RegOpenKeyExA" _
                              (ByVal hkey As Long, _
                              ByVal lpSubKey As String, _
                              ByVal ulOptions As Long, _
                              ByVal samDesired As Long, _
                              phkResult As Long) As Long

Declare Function RegCreateKeyEx Lib "advapi32.dll" Alias "RegCreateKeyExA" _
                              (ByVal hkey As Long, _
                              ByVal lpSubKey As String, _
                              ByVal Reserved As Long, _
                              ByVal lpClass As String, _
                              ByVal dwOptions As Long, _
                              ByVal samDesired As Long, _
                              lpSecurityAttributes As Any, _
                              phkResult As Long, _
                              lpdwDisposition As Long) As Long

Declare Function RegQueryValueEx Lib "advapi32.dll" Alias "RegQueryValueExA" _
                              (ByVal hkey As Long, _
                              ByVal lpValueName As String, _
                              ByVal lpReserved As Long, _
                              lpType As Long, _
                              lpData As Any, _
                              lpcbData As Long) As Long

Declare Function RegEnumKeyEx Lib "advapi32.dll" Alias "RegEnumKeyExA" _
                              (ByVal hkey As Long, _
                              ByVal dwIndex As Long, _
                              ByVal lpName As String, _
                              lpcbName As Long, _
                              ByVal lpReserved As Long, _
                              ByVal lpClass As Any, _
                              lpcbClass As Long, _
                              lpftLastWriteTime As FILETIME) As Long

Declare Function RegDeleteKey Lib "advapi32.dll" Alias "RegDeleteKeyA" _
                              (ByVal hkey As Long, _
                              ByVal lpSubKey As String) As Long

Declare Function RegCloseKey Lib "advapi32.dll" (ByVal hkey As Long) As Long

' hKey values
Public Const HKEY_CLASSES_ROOT = &H80000000

' Value data value type for the RegQueryValueEx
' lpType and RegSetValueEx dwType params.
Public Const REG_SZ = 1     ' Unicode null terminated string

' "The operation was successfully completed." error code
Public Const ERROR_SUCCESS = 0

Public Enum StandardAccessTypes
'  saDELETE = &H10000
  READ_CONTROL = &H20000
'  WRITE_DAC = &H40000
'  WRITE_OWNER = &H80000
  SYNCHRONIZE = &H100000
'  STANDARD_RIGHTS_REQUIRED = &HF0000
  STANDARD_RIGHTS_READ = (READ_CONTROL)
'  STANDARD_RIGHTS_WRITE = (READ_CONTROL)
'  STANDARD_RIGHTS_EXECUTE = (READ_CONTROL)
'  STANDARD_RIGHTS_ALL = &H1F0000
'  SPECIFIC_RIGHTS_ALL = &HFFFF&
End Enum

' Registry Specific Access Rights.
Public Enum REG_AccessRights
  KEY_QUERY_VALUE = &H1
'  KEY_SET_VALUE = &H2
  KEY_CREATE_SUB_KEY = &H4
  KEY_ENUMERATE_SUB_KEYS = &H8
  KEY_NOTIFY = &H10
'  KEY_CREATE_LINK = &H20
  KEY_READ = ((STANDARD_RIGHTS_READ Or KEY_QUERY_VALUE Or KEY_ENUMERATE_SUB_KEYS Or KEY_NOTIFY) And (Not SYNCHRONIZE))
'  KEY_WRITE = ((STANDARD_RIGHTS_WRITE Or KEY_SET_VALUE Or KEY_CREATE_SUB_KEY) And (Not SYNCHRONIZE))
'  KEY_EXECUTE = ((KEY_READ) And (Not SYNCHRONIZE))
'  KEY_ALL_ACCESS = ((STANDARD_RIGHTS_ALL Or KEY_QUERY_VALUE Or KEY_SET_VALUE Or KEY_CREATE_SUB_KEY Or KEY_ENUMERATE_SUB_KEYS Or KEY_NOTIFY Or KEY_CREATE_LINK) And (Not SYNCHRONIZE))
End Enum
'

' Retrieves registry value strings.
'   hKey            - HKEY_* registry constant
'   sSubkey       - subkey reg path string the desired value lives in
'   sValueName - value name string

' Rtns True if either both sSubkey and sValueName exist,
' or if sValueName is empty, i.e. is either "" or is (Default).

Public Function Reg_GetStringValue(hkey As Long, _
                                                          sSubkey As String, _
                                                          sValueName As String) As String
  Dim hSubkey As Long
  Dim cbData As Long
  Dim sData As String
  
  If (RegOpenKeyEx(hkey, sSubkey, 0, KEY_QUERY_VALUE, hSubkey) = ERROR_SUCCESS) Then
    
    ' Get the the len of the data for sData, includes null char!!!
    sData = vbNullChar   ' must be null, cbData = 0!!!!
    Call RegQueryValueEx(hSubkey, sValueName, 0, REG_SZ, ByVal sData, cbData)
    sData = String$(cbData, 0)
    
    If RegQueryValueEx(hSubkey, sValueName, 0, _
                                     REG_SZ, ByVal sData, cbData) = ERROR_SUCCESS Then
      ' Cleanup the string, sData will be empty if the data is either "" or "(value not set)" !
      Reg_GetStringValue = Left$(sData, cbData)
    End If   ' RegQueryValueEx
    
    Call RegCloseKey(hSubkey)
  End If   ' RegOpenKeyEx
  
End Function

Public Function Reg_CreateSubkey(hkey As Long, sSubkey As String) As Boolean
  Dim hSubkey As Long
  Dim dwResult As Long
  
  ' If sSubkey exists, its opened and dwResult = 2, if not, its created and opened, dwResult = 1.
  If (RegCreateKeyEx(hkey, sSubkey, 0, 0, 0, KEY_CREATE_SUB_KEY, _
                                  ByVal 0&, hSubkey, dwResult) = ERROR_SUCCESS) Then
    Reg_CreateSubkey = True
    Call RegCloseKey(hSubkey)
  End If
  
End Function

Public Function Reg_IsValidSubkey(hkey As Long, sSubkey As String) As Boolean
  Dim hSubkey As Long
  Dim dwResult As Long
  
  If (RegOpenKeyEx(hkey, sSubkey, 0, KEY_QUERY_VALUE, hSubkey) = ERROR_SUCCESS) Then
    Reg_IsValidSubkey = True
    Call RegCloseKey(hSubkey)
  End If
  
End Function

Public Function Reg_SetControlSafety(sCLSID As String, dwCmd As ComponentSafetyCmds) As Boolean
  Dim sImpCat As String
  Dim fRtn As Boolean
  
  ' ensure that sCLSID does in fact belong to a currently registered control.
  If Reg_IsValidSubkey(HKEY_CLASSES_ROOT, "CLSID\" & sCLSID & "\ProgID") Then
    sImpCat = "CLSID\" & sCLSID & "\Implemented Categories\"
    
    Select Case dwCmd
      Case cscMark
        fRtn = Reg_CreateSubkey(HKEY_CLASSES_ROOT, sImpCat & CATID_SafeForInitializing)
        fRtn = fRtn And Reg_CreateSubkey(HKEY_CLASSES_ROOT, sImpCat & CATID_SafeForScripting)
      Case cscUnmark
        fRtn = (RegDeleteKey(HKEY_CLASSES_ROOT, sImpCat & CATID_SafeForInitializing) = ERROR_SUCCESS)
        fRtn = fRtn And (RegDeleteKey(HKEY_CLASSES_ROOT, sImpCat & CATID_SafeForScripting) = ERROR_SUCCESS)
      Case cscIsMarked
        fRtn = Reg_IsValidSubkey(HKEY_CLASSES_ROOT, sImpCat & CATID_SafeForInitializing)
        fRtn = fRtn And Reg_IsValidSubkey(HKEY_CLASSES_ROOT, sImpCat & CATID_SafeForScripting)
    End Select

    Reg_SetControlSafety = fRtn
  End If   ' Reg_IsValidSubkey
  
End Function

Public Function CLSIDStrFromProgID(sProgID As String) As String
  Dim clsid As GUID
  Dim nChars As Long
  Dim sCLSID As String
  
  ' get the CLSID from the prog ID
  If (CLSIDFromProgID(StrConv(sProgID, vbUnicode), clsid) = S_OK) Then
    ' get the CLSID string from the CLSID
    sCLSID = Space$(MAX_GUID * 2)
    ' wants a Unicode buffer, nChars includes null terminator
    nChars = StringFromGUID2(clsid, ByVal sCLSID, MAX_GUID)
    If nChars Then
      CLSIDStrFromProgID = Left$(StrConv(sCLSID, vbFromUnicode), nChars - 1)
    End If
  End If

End Function

Public Function LVFindItemString(ByVal objLV As ListView, _
                                                      ByVal s As String, _
                                                      ByVal iStart As Long, _
                                                      li As ListItem, _
                                                      nSubItem As Long) As Boolean
'                                                      iSeachColumn As Long, _

  Dim nItems As Long
  Dim nCols As Long
  Dim i As Long
  Dim j As Long
  
  With objLV
    nItems = .ListItems.Count
    nCols = .ColumnHeaders.Count
    
    If (iStart >= 1) And (iStart <= nItems) Then
      
      For i = iStart To nItems
        If InStr(1, .ListItems(i), s, vbTextCompare) Then
          Set li = .ListItems(i)
          LVFindItemString = True
          Exit Function
        
        Else
          For j = 1 To nCols - 1   ' no SubItem.Index for main ListItem
            If InStr(1, .ListItems(i).SubItems(j), s, vbTextCompare) Then
              Set li = .ListItems(i)
              nSubItem = j
              LVFindItemString = True
              Exit Function
            End If
          Next   ' j
        
        End If   ' StrComp
      Next   ' i
    
    End If   ' (iStart >= 1) And (iStart <= nItems)
  End With   ' ListView1
  
End Function
