unit mShellDefs;

// DISCLAIMER!
// ==================================================================
// This source code has been automatically generated.
// It still requires extensive manual fixing, debugging and testing.
// IT IS NOT GUARANTEED TO COMPILE
// AND MUCH LESS TO BEHAVE EXACTLY LIKE THE ORIGINAL SOURCE CODE.
// Use it at your sole risk and discretion.
//

interface

  const
    // Brought to you by Brad Martinez
    //   http://members.aol.com/btmtz/vb
    //   http://www.mvps.org/ccrp
    //
    // Code was written in and formatted for 8pt MS San Serif
    //
    // Note that "IShellFolder Extended Type Library v1.1" (ISHF_Ex.tlb)
    // included with this project, must be present and correctly registered
    // on your system, and referenced by this project, to allow use of the
    // IShellFolder, IContextMenu and IMalloc interfaces.
    // ====================================================
    // Defined as an HRESULT that corresponds to S_OK.
    NOERROR : smallint = 0; 
    // GetItemID item ID retrieval constants
    GIID_FIRST : smallint = 1; 
    GIID_LAST : smallint = -1; 

  //
  // ====================================================
  // item ID (pidl) structs, just for reference
  //
  //' item identifier (relative pidl), allocated by the shell
  //Type SHITEMID
  //  cb As Integer        ' size of struct, including cb itself
  //  abID(0) As Byte    ' variable length item identifier
  //End Type
  //
  //' fully qualified pidl
  //Type ITEMIDLIST
  //  mkid As SHITEMID  ' list of item identifers, packed into SHITEMID.abID
  //End Type
  //
  // Returns a reference to the IMalloc interface.
  function MemAllocator : IMalloc;
  // ====== Begin pidl procs ===============================
  // Determines if the specified pidl is the desktop folder's pidl.
  // Returns True if the pidl is the desktop's pidl, returns False otherwise.
  // The desktop pidl is only a single item ID whose value is 0 (the 2 byte
  // zero-terminator, i.e. SHITEMID.abID is empty). Direct descendents of
  // the desktop (My Computer, Network Neighborhood) are absolute pidls
  // (relative to the desktop) also with a single item ID, but contain values
  // (SHITEMID.abID > 0). Drive folders have 2 item IDs, children of drive
  // folders have 3 item IDs, etc. All other single item ID pidls are relative to
  // the shell folder in which they reside (just like a relative path).
  function IsDesktopPIDL( var pidl : longint ) : boolean;
  // Returns the size in bytes of the first item ID in a pidl.
  // Returns 0 if the pidl is the desktop's pidl or is the last
  // item ID in the pidl (the zero terminator), or is invalid.
  function GetItemIDSize( pidl : longint ) : smallint;
  // Returns the count of item IDs in a pidl.
  function GetItemIDCount( pidl : longint ) : smallint;
  // Returns a pointer to the next item ID in a pidl.
  // Returns 0 if the next item ID is the pidl's zero value terminating 2 bytes.
  function GetNextItemID( pidl : longint ) : longint;
  // If successful, returns the size in bytes of the memory occcupied by a pidl,
  // including it's 2 byte zero terminator. Returns 0 otherwise.
  function GetPIDLSize( pidl : longint ) : smallint;
  // Copies and returns the specified item ID from a complex pidl
  //   pidl -    pointer to an item ID list from which to copy
  //   nItem - 1-based position in the pidl of the item ID to copy
  // If successful, returns a new item ID (single-element pidl)
  // from the specified element positon. Returns 0 on failure.
  // If nItem exceeds the number of item IDs in the pidl,
  // the last item ID is returned.
  // (calling proc is responsible for freeing the new pidl)
  function GetItemID( pidl : longint; nItem : smallint ) : longint;
  // Returns an absolute pidl (relative to the desktop) from a valid file system
  // path only (i.e. not from a display name).
  //   hwndOwner - handle of window that will own any displayed msg boxes
  //   sPath           - fully qualified path whose pidl is to be returned
  // If successful, the path's pidl is returned, otherwise 0 is returned.
  // (calling proc is responsible for freeing the pidl)
  function GetPIDLFromPath( var hwndOwner : longint; var sPath : widestring ) : longint;
  //
  // ====== End pidl procs ===============================
  //
  // Returns a reference to the parent IShellFolder of the last
  // item ID in the specified fully qualified pidl.
  // If pidlFQ is zero, or a relative (single item) pidl, then the
  // desktop's IShellFolder is returned.
  // If an unexpected error occurs, the object value Nothing is returned.
  function GetParentIShellFolder( pidlFQ : longint ) : IShellFolder;

  // Retrieves the IShellFolder interface for the desktop folder.
  // Returns NOERROR if successful or an OLE-defined error result otherwise.
  function SHGetDesktopFolder( var ppshf : IShellFolder ) : longint; stdcall; external 'shell32';
  // Retrieves a pointer to the shell's IMalloc interface.
  // Returns NOERROR if successful or or E_FAIL otherwise.
  function SHGetMalloc( var ppMalloc : IMalloc ) : longint; stdcall; external 'shell32';

implementation

  function MemAllocator : IMalloc;
    const
      {$WRITEABLECONST ON}
      im : IMalloc = ;
      {$WRITEABLECONST OFF}
    begin
      // SHGetMalloc should just get called once as the 'im'
      // variable stays in scope while the project is running...
      if ( im = nil )
        then
          begin
            SHGetMalloc( im );
          end;
      result := im;
    end;

  function IsDesktopPIDL( var pidl : longint ) : boolean;
    begin
      // The GetItemIDSize() call will also return 0 if pidl = 0
      if pidl
        then
          begin
            result := ( GetItemIDSize( pidl ) = 0 );
          end;
    end;

  function GetItemIDSize( pidl : longint ) : smallint;
    begin
      // If we try to access memory at address 0 (NULL), then it's bye-bye...
      if pidl
        then
          begin
            MoveMemory( result, pidl, 2 );
          end;
    end;

  function GetItemIDCount( pidl : longint ) : smallint;
    var
      nItems : smallint; 
    begin
      // If the size of an item ID is 0, then it's the zero
      // value terminating item ID at the end of the pidl.
      while GetItemIDSize( pidl ) do
        begin
          pidl := GetNextItemID( pidl );
          nItems := nItems + 1;
        end;
      result := nItems;
    end;

  function GetNextItemID( pidl : longint ) : longint;
    var
      cb : smallint;  // SHITEMID.cb, 2 bytes
    begin
      cb := GetItemIDSize( pidl );
      // Make sure it's not the zero value terminator.
      if cb
        then
          begin
            result := pidl + cb;
          end;
    end;

  function GetPIDLSize( pidl : longint ) : smallint;
    var
      cb : smallint; 
    label
      Out;
    begin
      // Error handle in case we get a bad pidl and overflow cb.
      // (most item IDs are roughly 20 bytes in size, and since an item ID represents
      // a folder, a pidl can never exceed 260 folders, or 5200 bytes).
      //ON ERROR
      if pidl
        then
          begin
            while pidl do
              begin
                cb := cb + GetItemIDSize( pidl );
                pidl := GetNextItemID( pidl );
              end;
            // Add 2 bytes for the zero terminating item ID
            result := cb + 2;
          end;
      Out:
    end;

  function GetItemID( pidl : longint; nItem : smallint ) : longint;
    var
      nCount : smallint; 
      i : smallint; 
      cb : smallint; 
      pidlNew : longint; 
    begin
      nCount := GetItemIDCount( pidl );
      if ( nItem > nCount ) or ( nItem = GIID_LAST )
        then
          begin
            nItem := nCount;
          end;
      // GetNextItemID returns the 2nd item ID
      for i := 1 to nItem - 1 do
        begin
          pidl := GetNextItemID( pidl );
        end;
      // Get the size of the specified item identifier.
      // If cb = 0 (the zero terminator), the we'll return a desktop pidl, proceed
      cb := GetItemIDSize( pidl );
      // Allocate a new item identifier list.
      pidlNew := MemAllocator.Alloc( cb + 2 );
      if pidlNew
        then
          begin
            // Copy the specified item identifier.
            // and append the zero terminator.
            MoveMemory( pidlNew, pidl, cb );
            MoveMemory( pidlNew + cb, 0, 2 );
            result := pidlNew;
          end;
    end;

  function GetPIDLFromPath( var hwndOwner : longint; var sPath : widestring ) : longint;
    var
      isfDesktop : IShellFolder; 
      pchEaten : longint; 
      pidl : longint; 
    begin
      if ( SHGetDesktopFolder( isfDesktop ) = NOERROR )
        then
          begin
            if ( isfDesktop.ParseDisplayName( hwndOwner, 0, StrConv( sPath, vbUnicode ), pchEaten, pidl, 0 ) = NOERROR )
              then
                begin
                  result := pidl;
                end;
          end;
    end;

  function GetParentIShellFolder( pidlFQ : longint ) : IShellFolder;
    var
      nCount : smallint; 
      i : smallint; 
      isf : IShellFolder; 
      pidlRel : longint; 
      IID_IShellFolder : GUID; 
    label
      Out;
    begin
      //ON ERROR
      nCount := GetItemIDCount( pidlFQ );
      // If pidlFQ is 0 and is not the desktop's pidl...
      if ( nCount = 0 ) and ( IsDesktopPIDL( pidlFQ ) = false )
        then
          begin
            RunError( 1 );
          end;
      // Get the desktop's IShellfolder first.
      if ( SHGetDesktopFolder( isf ) = NOERROR )
        then
          begin
            // Fill the IShellFolder interface ID, {000214E6-000-000-C000-000000046}
            with IID_IShellFolder do
              begin
                Data1 := $000214E6;
                Data4[0] := $000000C0;
                Data4[7] := $00000046;
              end;
            // Walk through the pidl and bind all the way to it's *2nd to last* item ID.
            for i := 1 to nCount - 1 do
              begin
                // Get the next item ID in the pidl (child of the current IShellFolder)
                pidlRel := GetItemID( pidlFQ, i );
                // Bind to the item current ID's folder and get it's IShellFolder
                if ( isf.BindToObject( pidlRel, 0, IID_IShellFolder, isf ) <> NOERROR )
                  then
                    begin
                      RunError( 1 );
                    end;
                // Free the current item ID and zero it
                MemAllocator.Free( pidlRel );
                pidlRel := 0;
              end;
          end; // SHGetDesktopFolder(isf) = NOERROR
      Out:
      if pidlRel
        then
          begin
            MemAllocator.Free( pidlRel );
          end;
      // Return a reference to the IShellFolder
      result := isf;
    end;

end.
